@@ -1,4 +1,4 @@
-# $Id: Navigation.rb,v 1.1.1.1 2004/04/04 15:22:49 toki Exp $
+# $Id: Navigation.rb,v 1.3 2004/10/14 15:25:11 toki Exp $
 
 class Navigation < WPM::PageContext
   include WPM::Escape
@@ -60,6 +60,10 @@ class Navigation < WPM::PageContext
     @page == 'Logging' || @page == 'AddLogFile'
   end
 
+  def access_log?
+    @page == 'AccessLog' || @page == 'AddAccessLog'
+  end
+
   def error?
     @page == 'Error'
   end
@@ -67,4 +71,8 @@ class Navigation < WPM::PageContext
   def administrator?
     @page == 'Administrator' || @page == 'AdminPassword'
   end
+
+  def about?
+    @page == 'About'
+  end
 end
