@@ -1,4 +1,4 @@
-# $Id: wpm.rb,v 1.1.1.1 2004/04/04 15:22:49 toki Exp $
+# $Id: wpm.rb,v 1.13 2004/10/27 13:18:41 toki Exp $
 # -
 # Web Page Maker - web application framework like CGIKit
 
@@ -59,7 +59,7 @@ module WPM
       when 2
 	ns_uri, name = args
       else
-	raise 'invalid arguments: ' + args.map{ |a| a.inspect }.join(', ')
+	raise 'invalid arguments: ' + args.map{|a| a.inspect }.join(', ')
       end
 
       if (@map.include? ns_uri) then
@@ -376,13 +376,13 @@ module WPM
       CHAR_SYMBOL[utf8] = sym
     end
 
-    CHAR_PATTERN = Regexp.compile(CHARACTER_ENTITY.map{ |sym, code, utf8|
+    CHAR_PATTERN = Regexp.compile(CHARACTER_ENTITY.map{|sym, code, utf8|
 				    Regexp.quote(utf8, 'u')
 				  }.join('|'),
 				  false, 'u')
 
     def escape_specials(raw_string, mode=:number)
-      raw_string.gsub(CHAR_PATTERN) { |utf8|
+      raw_string.gsub(CHAR_PATTERN) {|utf8|
 	case (mode)
 	when :number
 	  '&#' + CHAR_CODE[utf8].to_s + ';'
@@ -398,8 +398,8 @@ module WPM
 
   module Escape
     def escapeURL(raw_string)
-      url_string = raw_string.gsub(/[^ _0-9A-Za-z.-]/) { |special|
-	special.unpack('C*').map{ |c|
+      url_string = raw_string.gsub(/[^ _0-9A-Za-z.-]/) {|special|
+	special.unpack('C*').map{|c|
 	  format('%%%02X', c)
 	}.join
       }
@@ -410,7 +410,7 @@ module WPM
 
     def unescapeURL(url_string)
       raw_string = url_string.gsub(/\+/, ' ')
-      raw_string.gsub!(/%([0-9A-Fa-f][0-9A-Fa-f])/) { |special| $1.hex.chr }
+      raw_string.gsub!(/%([0-9A-Fa-f][0-9A-Fa-f])/) {|special| $1.hex.chr }
       raw_string
     end
     module_function :unescapeURL
@@ -465,7 +465,7 @@ module WPM
   class MessageManipulator < Driver
     include Escape
 
-    NO_FILTER = proc{ |messg_body|
+    NO_FILTER = proc{|messg_body|
       messg_body
     }
 
@@ -666,7 +666,7 @@ module WPM
       method = environ['REQUEST_METHOD']
       page = environ['PATH_INFO']
       if (method != 'POST') then
-	query = params.map{ |name, value| "#{name}=#{value}" }.join('&')
+	query = params.map{|name, value| "#{name}=#{value}" }.join('&')
 	unless (query.empty?) then
 	  page += '?'
 	  page += query
@@ -680,7 +680,7 @@ module WPM
       end
 
       log = "#{timestamp}: [#{rhost}] #{method} #{page} - #{status}: #{user_agent} < #{referer}\n"
-      File.open(@filename, 'a') { |output|
+      File.open(@filename, 'a') {|output|
 	output.print @output_filter.call(log)
       }
 
@@ -689,6 +689,28 @@ module WPM
   end
 
   module CheckUtil
+    def check_integer(string)
+      case (string.strip)
+      when /^[+-]?\d+$/
+	return
+      else
+	raise "not a integer: #{string.inspect}"
+      end
+    end
+    module_function :check_integer
+
+    def check_float(string)
+      case (string.strip)
+      when /^[+-]?\d+(\.\d+)?$/, /^[+-]?\.\d+$/
+	return
+      when /^[+-]?\d+(\.\d+)?[Ee][+-]?\d+$/, /^[+-]?\.\d+[Ee][+-]?\d+$/
+	return
+      else
+	raise "not a float: #{string.inspect}"
+      end
+    end
+    module_function :check_float
+
     def check_number(string)
       if (string !~ /^(\+|\-)?\d+$/) then
 	raise "not a number: #{string.inspect}"
@@ -771,8 +793,7 @@ module WPM
   # (1) call WPM::Component#compile -> building a tree of components.
   # (2) call WPM::Component#setup -> setup each component.
   # (3) call WPM::Component#run -> executing actions at each component.
-  # (4) call WPM::Component#follow -> following components of imported pages.
-  # (5) call WPM::Component#make_page -> building a page from a tree of components.
+  # (4) call WPM::Component#make_page -> building a page from a tree of components.
   #
   class Component
     include Escape
@@ -824,15 +845,15 @@ module WPM
     def compile
     end
 
+    def call_reset
+    end
+
     def setup
     end
 
     def run
     end
 
-    def follow
-    end
-
     def make_page
       yield(escapeHTML(self.class.to_s))
       nil
@@ -879,6 +900,14 @@ module WPM
       nil
     end
 
+    def call_reset
+      for component in @child_components
+	component.call_reset
+      end
+
+      nil
+    end
+
     def setup
       for component in @child_components
 	component.setup
@@ -895,14 +924,6 @@ module WPM
       nil
     end
 
-    def follow
-      for component in @child_components
-	component.follow
-      end
-
-      nil
-    end
-
     def each
       for component in @child_components
 	yield(component)
@@ -1033,7 +1054,7 @@ module WPM
     def initialize(widget_id, context)
       @context = context
       @widget_id = widget_id
-      @call_index = -1
+      @call_index = nil
       @call_index_setter = nil
     end
 
@@ -1044,11 +1065,6 @@ module WPM
     attr_reader :widget_id
     attr_reader :call_index
 
-    def call_index=(new_index)
-      @call_index_setter.call(new_index) if @call_index_setter
-      new_index
-    end
-
     def call_count
       @call_index += 1
       @call_index_setter.call(@call_index) if @call_index_setter
@@ -1188,7 +1204,7 @@ module WPM
 	attr_writer = "#{prop_value}=".intern
       end
 
-      proc{ |*args|
+      proc{|*args|
 	case (args.length)
 	when 0
 	  context.__send__(attr_reader)
@@ -1199,7 +1215,7 @@ module WPM
 	    raise PropertyError, "read only accessor: #{attr_reader}"
 	  end
 	else
-	  raise PropertyError, 'invalid accessor arguments: ' + args.map{ |a| a.inspect }.joni(', ') + ": at #{@widget_id.inspect}"
+	  raise PropertyError, 'invalid accessor arguments: ' + args.map{|a| a.inspect }.joni(', ') + ": at #{@widget_id.inspect}"
 	end
       }
     end
@@ -1246,7 +1262,7 @@ module WPM
   # (2) binding phase.
   #   (2.1) call WPM::Component#compile -> building a tree of components.
   #     (2.1.1) call WPM::WidgetProperties#create_widget -> creating a widget having properties.
-  #   (2.2) call WPM::Component#setup -> setup each component.
+  #   (2.2) call WPM::Widget#init -> initialize each widget.
   #   (2.3) call WPM::Widget#invoke -> executing an action at a widget.
   #   (2.4) call WPM::Widget#view -> building widget view.
   #
@@ -1274,6 +1290,9 @@ module WPM
       component_name + '.' + call_index.to_s
     end
 
+    def init
+    end
+
     def invoke
     end
 
@@ -1282,11 +1301,16 @@ module WPM
       nil
     end
 
-    def run
-      invoke
+    def setup
+      @properties.call_count
+      init
 
-      # traversing child components.
-      super
+      nil
+    end
+
+    def run
+      @properties.call_count
+      invoke
 
       nil
     end
@@ -1340,6 +1364,13 @@ module WPM
       @curr_value = nil
     end
 
+    def call_reset
+      for widget_id, properties in @properties_map
+	properties.call_reset
+      end
+      nil
+    end
+
     def create_widget(widget_id)
       unless (@properties_map.include? widget_id) then
 	raise "not found a widget with id: #{widget_id.inspect}"
@@ -1505,6 +1536,22 @@ module WPM
     end
     private :execute
 
+    def init
+      execute do |component|
+	component.setup
+      end
+
+      nil
+    end
+
+    def invoke
+      execute do |component|
+	component.run
+      end
+
+      nil
+    end
+
     def view
       execute do |component|
 	component.make_page do |page_text|
@@ -1561,13 +1608,36 @@ module WPM
   end
 
   class RepeatWidget < Widget
-    def view
+    def execute
       for i in 1..(@properties.times)
 	@properties.index = i if @properties.has_index?
 	for component in @child_components
-	  component.make_page do |page_text|
-	    yield(page_text)
-	  end
+	  yield(component)
+	end
+      end
+    end
+    private :execute
+
+    def init
+      execute do |component|
+	component.setup
+      end
+
+      nil
+    end
+
+    def invoke
+      execute do |component|
+	component.run
+      end
+
+      nil
+    end
+
+    def view
+      execute do |component|
+	component.make_page do |page_text|
+	  yield(page_text)
 	end
       end
 
@@ -1659,6 +1729,22 @@ module WPM
     end
     private :execute
 
+    def init
+      execute do |component|
+	component.setup
+      end
+
+      nil
+    end
+
+    def invoke
+      execute do |component|
+	component.run
+      end
+
+      nil
+    end
+
     def view
       execute do |component|
 	component.make_page do |page_text|
@@ -1730,16 +1816,13 @@ module WPM
     end
     private :parent_page
 
-    def follow
+    def compile
       @another_page = parent_page.new_page(@properties.page)
       @another_page.parent_component = self
-      @another_page.context_hook{ |context|
+      @another_page.context_hook{|context|
 	@properties.setup_import_params(context)
       }
       @another_page.compile
-      @another_page.setup
-      @another_page.run
-      @another_page.follow
 
       # traversing child components.
       super
@@ -1747,6 +1830,22 @@ module WPM
       nil
     end
 
+    def call_reset
+      @another_page.call_reset
+      super			# traversing child components.
+      nil
+    end
+
+    def init
+      @another_page.setup
+      nil
+    end
+
+    def invoke
+      @another_page.run
+      nil
+    end
+
     def view
       @another_page.make_page do |page_text|
 	yield(page_text)
@@ -1790,6 +1889,22 @@ module WPM
     end
     private :execute
 
+    def init
+      execute do |component|
+	component.setup
+      end
+
+      nil
+    end
+
+    def invoke
+      execute do |component|
+	component.run
+      end
+
+      nil
+    end
+
     def view
       execute do |component|
 	component.make_page do |page_text|
@@ -1983,19 +2098,30 @@ module WPM
   end
 
   class HyperlinkWidget < Widget
+    def init
+      for component in @child_components
+	component.setup
+      end
+
+      nil
+    end
+
     def invoke
       if (@properties.has_action?) then
 	if (action_id = driver.params['action']) then
 	  action_id =~ /^(\S+)\.(\d+)$/ or raise WidgetError, "invalid action-id syntax: #{action_id.inspect}"
 	  component_name = $1
 	  call_index = $2.to_i
-	  if (component_name == self.component_name) then
-	    @properties.call_index = call_index
+	  if (call_index == @properties.call_index && component_name == self.component_name) then
 	    @properties.action
 	  end
 	end
       end
 
+      for component in @child_components
+	component.run
+      end
+
       nil
     end
 
@@ -2027,9 +2153,9 @@ module WPM
 	end
       end
 
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' href="' + escapeHTML(href) + '"'
-	attr_desc += ' target="' + escapeHTML(self.target) + '"' if @properties.has_target?
+	attr_desc += ' target="' + escapeHTML(@properties.target) + '"' if @properties.has_target?
 	yield('<a' + attr_desc + '>')
 	if (has_children?) then
 	  for component in @child_components
@@ -2096,7 +2222,7 @@ module WPM
 
   class ImageWidget < Widget
     def view
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' src="' + escapeHTML(@properties.src) + '"'
 	attr_desc += ' alt="' + escapeHTML(@properties.alt) + '"' if @properties.has_alt?
 	yield('<img' + attr_desc + ' />')
@@ -2171,7 +2297,7 @@ module WPM
 
   class FrameWidget < Widget
     def view
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' name="' + escapeHTML(@properties.name) + '"'
 	if (@properties.has_page?) then
 	  attr_desc += ' src="' + escapeHTML(driver.page_path(@properties.page)) + '"'
@@ -2225,6 +2351,7 @@ module WPM
   class FormWidget < Widget
     def compile
       @control = Hash.new
+      @params = driver.params
 
       # traversing child components.
       super
@@ -2233,85 +2360,47 @@ module WPM
     end
 
     attr_reader :control
+    attr_reader :params
 
-    def setup
-      form_params = driver.params
-      @call_params = Hash.new
-      for name, value in form_params
-	if (name =~ /^(\S+)\.(\d+)$/) then
-	  component_name = $1
-	  call_index = $2.to_i
-	  @call_params[component_name] = Hash.new unless (@call_params.include? component_name)
-	  @call_params[component_name][call_index] = value
-	end
+    def submit?
+      @params['_wpm_submit_'] == action_id
+    end
+
+    def init
+      for component in @child_components
+	component.setup
       end
 
-      @call_range = Hash.new
-      for name, value in form_params
-	if (name =~ /^call-range:(\S+)$/) then
-	  control_name = $1
-	  if (value =~ /^(\d+)-(\d+)$/) then
-	    call_first = $1.to_i
-	    call_last = $2.to_i
-	    @call_range[control_name] = Range.new(call_first, call_last)
-	  end
-	end
-      end
-
-      # traversing child components.
-      super
-
       nil
     end
 
-    attr_reader :call_params
-
-    def call_range(name)
-      if (@call_range.include? name) then
-	return @call_range[name]
-      else
-	return Range.new(0, 0, true)
+    def invoke
+      for component in @child_components
+	component.run
       end
+
+      nil
     end
 
     def view
-      @new_call_range = Hash.new
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	http_method = @properties.http_method
 	attr_desc += ' id="' + escapeHTML(action_id) + '"'
 	attr_desc += ' method="' + escapeHTML(http_method) + '"'
 	attr_desc += ' enctype="' + escapeHTML(@properties.enctype) + '"' if (http_method == 'post')
 	yield('<form' + attr_desc + '>')
-
+	yield('<div style="display: none">')
+	yield('<input type="hidden" name="_wpm_submit_" value="' + escapeHTML(action_id) + '" />')
+	yield('</div>')
 	for component in @child_components
 	  component.make_page do |page_text|
 	    yield(page_text)
 	  end
 	end
-
-	unless (@new_call_range.empty?) then
-	  yield('<div>')
-	  for call_name, (call_first, call_last) in @new_call_range
-	    hidden_param = '<input type="hidden"'
-	    hidden_param += ' name="' + escapeHTML("call-range:#{call_name}") + '"'
-	    hidden_param += ' value="' + escapeHTML("#{call_first}-#{call_last}") + '"'
-	    hidden_param += ' />'
-	    yield(hidden_param)
-	  end
-	  yield('</div>')
-	end
-
 	yield('</form>')
       }
 
-      nil
-    end
 
-    def set_call_range(control_widget)
-      call_name = control_widget.component_name
-      call_index = control_widget.call_index
-      @new_call_range[call_name] = [ call_index, call_index ] unless (@new_call_range.include? call_name)
-      @new_call_range[call_name][1] = call_index if (call_index > @new_call_range[call_name][1])
       nil
     end
   end
@@ -2401,12 +2490,24 @@ module WPM
   end
 
   class LabelWidget < FormElementWidget
+    def init
+      for component in @child_components
+	component.setup
+      end
+
+      nil
+    end
+
     def invoke
       control_widget = form.control[@properties.for_id]
       unless (control_widget) then
 	raise WidgetError, "not found a widget of #{@properties.for_id}"
       end
 
+      for component in @child_components
+	component.run
+      end
+
       nil
     end
 
@@ -2417,7 +2518,7 @@ module WPM
       end
       ctl_id = control_widget.component_name + '.' + @properties.call_index.to_s
 
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' for="' + escapeHTML(ctl_id) + '"'
 	yield('<label' + attr_desc + '>')
 	if (has_children?) then
@@ -2436,12 +2537,6 @@ module WPM
     end
   end
 
-  # memo: event flow.
-  # (1) call WPM::Component#compile -> building a tree of components.
-  # (2) call WPM::FormControlWidget#read_form -> reading form parameter.
-  # (3) call WPM::Widget#invoke -> executing an action at a widget.
-  # (4) call WPM::Widget#view -> building widget view.
-  #
   class FormControlWidget < FormElementWidget
     def compile
       form.control[widget_id] = self
@@ -2453,7 +2548,7 @@ module WPM
     end
 
     def make_attr_desc
-      super{ |attr_desc|
+      super{|attr_desc|
 	attr_desc += ' id="' + escapeHTML(action_id) + '"'
 	yield(attr_desc)
       }
@@ -2461,18 +2556,6 @@ module WPM
       nil
     end
     private :make_attr_desc
-
-    def read_form
-    end
-
-    def setup
-      read_form
-
-      # traversing child components.
-      super
-
-      nil
-    end
   end
 
   module DisabledWidgetProperty
@@ -2500,7 +2583,7 @@ module WPM
 
   module DisabledWidgetAttribute
     def make_attr_desc
-      super{ |attr_desc|
+      super{|attr_desc|
 	if (@properties.disabled) then
 	  attr_desc += ' disabled="disabled"'
 	end
@@ -2537,7 +2620,7 @@ module WPM
 
   module ReadOnlyWidgetAttribute
     def make_attr_desc
-      super{ |attr_desc|
+      super{|attr_desc|
 	if (@properties.readonly) then
 	  attr_desc += ' readonly="readonly"'
 	end
@@ -2559,6 +2642,7 @@ module WPM
       super
       @value = nil
       @size = nil
+      @data_type = 'string'
       nil
     end
 
@@ -2567,6 +2651,13 @@ module WPM
 	raise PropertyError, "required `value' property at #{widget_id.inspect}"
       end
 
+      case (@data_type)
+      when 'string', 'integer', 'float'
+	# nothing to do.
+      else
+	raise PropertyError, "unknown property value #{@data_type.inspect} of data-type"
+      end
+
       nil
     end
 
@@ -2576,6 +2667,8 @@ module WPM
 	@value = get_property(attr_map, value, 'accessor')
       when 'size'
 	@size = get_property(attr_map, value, 'number', 'accessor', 'eval')
+      when 'data-type'
+	@data_type = get_property(attr_map, value, 'string')
       else
 	super
       end
@@ -2584,11 +2677,31 @@ module WPM
     end
 
     def value
-      fetch_value('value', @value, ::String)
+      case (@data_type)
+      when 'string'
+	fetch_value('value', @value, ::String)
+      when 'integer'
+	fetch_value('value', @value, ::Integer).to_s
+      when 'float'
+	fetch_value('value', @value, ::Float).to_s
+      else
+	raise "internal error: #{@data_type.inspect}"
+      end
     end
 
     def value=(new_value)
-      @value.call(new_value)
+      case (@data_type)
+      when 'string'
+	@value.call(new_value)
+      when 'integer'
+	CheckUtil.check_integer(new_value)
+	@value.call(new_value.to_i)
+      when 'float'
+	CheckUtil.check_float(new_value)
+	@value.call(new_value.to_f)
+      else
+	raise "internal error: #{@data_type.inspect}"
+      end
     end
 
     def has_size?
@@ -2608,19 +2721,18 @@ module WPM
     include DisabledWidgetAttribute
     include ReadOnlyWidgetAttribute
 
-    def read_form
-      if (call_aset = form.call_params[component_name]) then
-	for call_index, call_value in call_aset.sort{ |a, b| a[0] <=> b[0] }
-	  @properties.call_index = call_index
-	  @properties.value = call_value
-	end
+    def init
+      form_params = form.params
+      action_id = self.action_id
+      if (form_params.include? action_id) then
+	@properties.value = form_params[action_id]
       end
 
       nil
     end
 
     def view
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' name="' + escapeHTML(action_id) + '"'
 	attr_desc += ' type="text"'
 	attr_desc += ' value="' + escapeHTML(@properties.value || '') + '"'
@@ -2687,19 +2799,18 @@ module WPM
     include DisabledWidgetAttribute
     include ReadOnlyWidgetAttribute
 
-    def read_form
-      if (call_aset = form.call_params[component_name]) then
-	for call_index, call_value in call_aset.sort{ |a, b| a[0] <=> b[0] }
-	  @properties.call_index = call_index
-	  @properties.value = call_value
-	end
+    def init
+      form_params = form.params
+      action_id = self.action_id
+      if (form_params.include? action_id) then
+	@properties.value = form_params[action_id]
       end
 
       nil
     end
 
     def view
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' name="' + escapeHTML(action_id) + '"'
 	attr_desc += ' type="password"'
 	attr_desc += ' size="' + escapeHTML(@properties.size.to_s) + '"' if @properties.has_size?
@@ -2788,28 +2899,31 @@ module WPM
     include ReadOnlyWidgetAttribute
     include DisabledWidgetAttribute
 
-    def read_form
-      for call_index in form.call_range(component_name)
-	@properties.call_index = call_index
-	if (call_aset = form.call_params[component_name]) then
-	  if (call_value = call_aset[call_index]) then
-	    @properties.checked = true if @properties.has_checked?
-	    @properties.selected = call_value if @properties.has_selected?
-	    next
+    def init
+      if (form.submit?) then
+	form_params = form.params
+	action_id = self.action_id
+
+	if (@properties.has_checked?) then
+	  if (form_params.include? action_id) then
+	    @properties.checked = true
+	  else
+	    @properties.checked = false
 	  end
 	end
 
-	@properties.checked = false if @properties.has_checked?
-	@properties.selected = nil if @properties.has_selected?
+	if (@properties.has_selected?) then
+	  if (form_params.include? action_id) then
+	    @properties.selected = form_params[action_id]
+	  end
+	end
       end
 
       nil
     end
 
     def view
-      form.set_call_range(self)
-
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' name="' + escapeHTML(action_id) + '"'
 	attr_desc += ' type="checkbox"'
 	attr_desc += ' value="' + escapeHTML((@properties.has_value?) ? @properties.value : '') + '"'
@@ -2894,7 +3008,7 @@ module WPM
     end
 
     def value
-      fetch_value('name', @value, ::String, true)
+      fetch_value('value', @value, ::String, true)
     end
 
     def has_selected?
@@ -2918,23 +3032,24 @@ module WPM
     include DisabledWidgetAttribute
     include ReadOnlyWidgetAttribute
 
-    def read_form
-      form_params = driver.params
-      for call_index in form.call_range(component_name)
-	@properties.call_index = call_index
+    def init
+      if (form.submit?) then
+	form_params = form.params
 	name = (@properties.has_name?) ? @properties.name : component_name
-	value = (@properties.has_value?) ? @properties.value : component_name + '.' + call_index.to_s
-	@properties.checked = form_params[name] == value if @properties.has_checked?
-	@properties.selected = form_params[name] if @properties.has_selected?
+	value = (@properties.has_value?) ? @properties.value : action_id
+	if (@properties.has_checked?) then
+	  @properties.checked = form_params[name] == value
+	end
+	if (@properties.has_selected?) then
+	  if (form_params.include? name) then
+	    @properties.selected = form_params[name]
+	  end
+	end
       end
-
-      nil
     end
 
     def view
-      form.set_call_range(self)
-
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' name="' + escapeHTML((@properties.has_name?) ? @properties.name : component_name) + '"'
 	attr_desc += ' type="radio"'
 	attr_desc += ' value="' + escapeHTML((@properties.has_value?) ? @properties.value : action_id) + '"'
@@ -3006,10 +3121,11 @@ module WPM
     include ReadOnlyWidgetAttribute
 
     def invoke
-      if (call_aset = form.call_params[component_name]) then
-	for call_index, call_value in call_aset.sort{ |a, b| a[0] <=> b[0] }
-	  @properties.call_index = call_index
-	  @properties.action if @properties.has_action?
+      if (@properties.has_action?) then
+	form_params = form.params
+	action_id = self.action_id
+	if (form_params.include? action_id) then
+	  @properties.action
 	end
       end
 
@@ -3017,7 +3133,7 @@ module WPM
     end
 
     def view
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' name="' + escapeHTML(action_id) + '"'
 	attr_desc += ' type="submit"'
 	attr_desc += ' value="' + escapeHTML(@properties.value) + '"' if @properties.has_value?
@@ -3069,7 +3185,7 @@ module WPM
     include ReadOnlyWidgetAttribute
 
     def view
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' type="reset"'
 	attr_desc += ' value="' + escapeHTML(@properties.value) + '"' if @properties.has_value?
 	yield('<input' + attr_desc + ' />')
@@ -3089,6 +3205,7 @@ module WPM
       super
       @value = nil
       @name = nil
+      @data_type = 'string'
       nil
     end
 
@@ -3097,6 +3214,13 @@ module WPM
 	raise PropertyError, "required `value' property at #{@widget_id.inspect}"
       end
 
+      case (@data_type)
+      when 'string', 'integer', 'float'
+	# nothing to do.
+      else
+	raise PropertyError, "unknown property value #{@data_type.inspect} of data-type"
+      end
+
       nil
     end
 
@@ -3106,6 +3230,8 @@ module WPM
 	@value = get_property(attr_map, value, 'accessor')
       when 'name'
 	@name = get_property(attr_map, value, 'string', 'accessor', 'eval')
+      when 'data-type'
+	@data_type = get_property(attr_map, value, 'string')
       else
 	super
       end
@@ -3114,11 +3240,31 @@ module WPM
     end
 
     def value
-      fetch_value('value', @value, ::String, true)
+      case (@data_type)
+      when 'string'
+	fetch_value('value', @value, ::String)
+      when 'integer'
+	fetch_value('value', @value, ::Integer).to_s
+      when 'float'
+	fetch_value('value', @value, ::Float).to_s
+      else
+	raise "internal error: #{@data_type.inspect}"
+      end
     end
 
     def value=(new_value)
-      @value.call(new_value)
+      case (@data_type)
+      when 'string'
+	@value.call(new_value)
+      when 'integer'
+	CheckUtil.check_integer(new_value)
+	@value.call(new_value.to_i)
+      when 'float'
+	CheckUtil.check_float(new_value)
+	@value.call(new_value.to_f)
+      else
+	raise "internal error: #{@data_type.inspect}"
+      end
     end
 
     def has_name?
@@ -3138,25 +3284,22 @@ module WPM
     include DisabledWidgetAttribute
     include ReadOnlyWidgetAttribute
 
-    def read_form
+    def init
+      form_params = form.params
       if (@properties.has_name?) then
-	if (value = driver.params[@properties.name]) then
-	  @properties.value = value
-	end
+	name = @properties.name
       else
-	if (call_aset = form.call_params[component_name]) then
-	  for call_index, call_value in call_aset.sort{ |a, b| a[0] <=> b[0] }
-	    @properties.call_index = call_index
-	    @properties.value = call_value
-	  end
-	end
+	name = action_id
+      end
+      if (form_params.include? name) then
+	@properties.value = form_params[name]
       end
 
       nil
     end
 
     def view
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' name="' + escapeHTML((@properties.has_name?) ? @properties.name : action_id) + '"'
 	attr_desc += ' type="hidden"'
 	attr_desc += ' value="' + escapeHTML(@properties.value) + '"'
@@ -3237,19 +3380,18 @@ module WPM
     include DisabledWidgetAttribute
     include ReadOnlyWidgetAttribute
 
-    def read_form
-      if (call_aset = form.call_params[component_name]) then
-	for call_index, call_value in call_aset.sort{ |a, b| a[0] <=> b[0] }
-	  @properties.call_index = call_index
-	  @properties.value = call_value
-	end
+    def init
+      form_params = form.params
+      action_id = self.action_id
+      if (form_params.include? action_id) then
+	@properties.value = form_params[action_id]
       end
 
       nil
     end
 
     def view
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' name="' + escapeHTML(action_id) + '"'
 	attr_desc += ' rows="' + escapeHTML(@properties.rows.to_s) + '"' if @properties.has_rows?
 	attr_desc += ' cols="' + escapeHTML(@properties.cols.to_s) + '"' if @properties.has_cols?
@@ -3330,11 +3472,12 @@ module WPM
   class SelectWidget < FormControlWidget
     include DisabledWidgetAttribute
 
-    def read_form
-      if (call_aset = form.call_params[component_name]) then
-	for call_index, call_value in call_aset.sort{ |a, b| a[0] <=> b[0] }
-	  @properties.call_index = call_index
-	  @properties.selected = call_value if @properties.has_selected?
+    def init
+      if (@properties.has_selected?) then
+	form_params = form.params
+	action_id = self.action_id
+	if (form_params.include? action_id) then
+	  @properties.selected = form_params[action_id]
 	end
       end
 
@@ -3342,7 +3485,7 @@ module WPM
     end
 
     def view
-      make_attr_desc{ |attr_desc|
+      make_attr_desc{|attr_desc|
 	attr_desc += ' name="' + escapeHTML(action_id) + '"'
 	attr_desc += ' size="' + escapeHTML(@properties.size.to_s) + '"' if @properties.has_size?
 	yield('<select' + attr_desc + '>')
@@ -3437,8 +3580,8 @@ module WPM
   class Loader
     def initialize(filename)
       @mod = Module.new
-      File.open(filename) { |input|
-	@mod.module_eval(input.read)
+      File.open(filename) {|input|
+	@mod.module_eval(input.read, filename)
       }
     end
 
@@ -3450,7 +3593,7 @@ module WPM
   class Page < Component
     include ComponentParent
 
-    NULL_PROC = proc{ |context|
+    NULL_PROC = proc{|context|
       # nothing to do.
     }
 
@@ -3533,10 +3676,10 @@ module WPM
       @widget_locator.extend(@xml_assist)
       assembler.extend(@xml_assist)
 
-      File.open(map_name) { |input|
+      File.open(map_name) {|input|
 	@widget_locator.read(input)
       }
-      File.open(xml_name) { |input|
+      File.open(xml_name) {|input|
 	assembler.read(input)
       }
 
@@ -3546,6 +3689,12 @@ module WPM
       nil
     end
 
+    def call_reset
+      @widget_locator.call_reset
+      super			# traversing child components.
+      nil
+    end
+
     def setup
       @context.init_context
       @context_hook.call(@context)
@@ -3683,9 +3832,11 @@ module WPM
 	  if (page_name = @driver.curr_page) then
 	    page = Page.new(page_name, @driver, @xml_assist, @base_dir, @page_option)
 	    page.compile
+	    page.call_reset
 	    page.setup
+	    page.call_reset
 	    page.run
-	    page.follow
+	    page.call_reset
 	    page.make_page do |page_text|
 	      abort_on_writing = true
 	      @driver.write(page_text) if (req_method != 'HEAD')
