@@ -1,10 +1,18 @@
 #!/usr/local/bin/ruby
-# $Id: test_DocumentTable.rb,v 1.1.1.1 2004/04/04 15:22:50 toki Exp $
+# $Id: test_DocumentTable.rb,v 1.3 2004/10/20 14:37:32 toki Exp $
 
 require 'rubyunit'
 require 'wpm'
 require 'rucy/document'
 
+# patch
+class Array
+  def swap(i, j)
+    self[i], self[j] = self[j], self[i]
+    nil
+  end
+end
+
 module TestRucy
   class TestDocumentTable < RUNIT::TestCase
     def setup
@@ -131,7 +139,7 @@ module TestRucy
 
       # setup WPM::PageContext class
       loader = WPM::Loader.new('../control/DocumentTable/DocumentTable.rb')
-      @DocumentTable = loader.const_get('DocumentTable').new(self, :control => self, :factory => self)
+      @DocumentTable = loader.const_get('DocumentTable').new(self, { :control => self, :factory => self })
       assert((@DocumentTable.is_a? WPM::PageContext))
       @DocumentTable.init_context
       @DocumentTable.table_source = self
