@@ -1,5 +1,5 @@
 #!/usr/local/bin/ruby
-# $Id: test_EditProperties.rb,v 1.4 2004/04/21 15:09:31 toki Exp $
+# $Id: test_EditProperties.rb,v 1.5 2004/10/13 15:21:27 toki Exp $
 
 require 'rubyunit'
 require 'wpm'
@@ -125,7 +125,7 @@ module TestRucy
 
       # setup WPM::PageContext class
       loader = WPM::Loader.new('../control/EditProperties/EditProperties.rb')
-      @EditProperties = loader.const_get('EditProperties').new(self, :control => self, :pw_enc => self)
+      @EditProperties = loader.const_get('EditProperties').new(self, { :control => self, :pw_enc => self })
       assert((@EditProperties.is_a? WPM::PageContext))
       @EditProperties.init_context
       @EditProperties.properties_source = self
