@@ -1,40 +1,11 @@
-# $Id: test_cgi.rb,v 1.1.1.1 2004/04/04 15:22:50 toki Exp $
-
-=begin
-Copyright (c) 2001-2004
-	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.
-
-Redistribution and use in source and binary forms, with or without modification,
-are permitted provided that the following conditions are met:
-
-  1.  Redistributions of source code must retain the above copyright notice,
-      this list of conditions and the following disclaimer.
-
-  2.  Redistributions in binary form must reproduce the above copyright notice,
-      this list of conditions and the following disclaimer in the documentation
-      and/or other materials provided with the distribution.
-
-  3.  The name of the author may not be used to endorse or promote products
-      derived from this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
-WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
-SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
-OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
-IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
-OF SUCH DAMAGE.
-=end
+# $Id: test_cgi.rb,v 1.4 2004/09/29 14:43:08 toki Exp $
 
 require 'rubyunit'
 require 'pseudo_req_res'
 require 'pseudo_io'
 require 'rucy/request'
 require 'rucy/response'
-require 'rucy/cgi'
+require 'rucy/loader'
 
 module TestRucy
   module TestCGI
@@ -68,15 +39,18 @@ module TestRucy
 
     def setup
       super
-      @cgi = Rucy::CGIDocument.new(CGI_DIR, 'ruby')
+      @loader = Rucy::SimpleLoader.new
+      @loader.load('../mod_docs/cgi.rb')
+      @CGIDocument = @loader.const_get('CGIDocument')
+      @cgi = @CGIDocument.new(CGI_DIR, 'ruby')
     end
 
     def test_CGIDocument_quote_sh
-      assert_equal('"Hello world."', Rucy::CGIDocument.quote_sh('Hello world.'))
-      assert_equal('"\\""', Rucy::CGIDocument.quote_sh('"'))
-      assert_equal('"\\$"', Rucy::CGIDocument.quote_sh('$'))
-      assert_equal('"\\`"', Rucy::CGIDocument.quote_sh('`'))
-      assert_equal('"\\\\"', Rucy::CGIDocument.quote_sh('\\'))
+      assert_equal('"Hello world."', @CGIDocument.quote_sh('Hello world.'))
+      assert_equal('"\\""', @CGIDocument.quote_sh('"'))
+      assert_equal('"\\$"', @CGIDocument.quote_sh('$'))
+      assert_equal('"\\`"', @CGIDocument.quote_sh('`'))
+      assert_equal('"\\\\"', @CGIDocument.quote_sh('\\'))
     end
 
     def test_cgi
@@ -326,7 +300,10 @@ module TestRucy
 
     def setup
       super
-      @cgi = Rucy::CGIDocument.new('cgi', 'ruby', true)
+      @loader = Rucy::SimpleLoader.new
+      @loader.load('../mod_docs/cgi.rb')
+      @CGIDocument = @loader.const_get('CGIDocument')
+      @cgi = @CGIDocument.new('cgi', 'ruby', true)
     end
 
     def test_cgi
@@ -555,4 +532,50 @@ module TestRucy
       assert_equal("Hello world.\n", @messg_body)
     end
   end
+
+  class TestCGIDocumentFactory < RUNIT::TestCase
+    def setup
+      @loader = Rucy::SimpleLoader.new
+      @loader.load('../mod_docs/cgi.rb')
+      @CGIDocumentFactory = @loader.const_get('CGIDocumentFactory')
+      @factory = @CGIDocumentFactory.instance
+    end
+
+    def test_set_doc_option
+      @factory.set_doc_option(Hash.new)
+    end
+
+    def test_doc_name
+      assert_equal('CGI', @factory.doc_name)
+    end
+
+    def test_doc_args
+      args = @factory.doc_args
+      assert_equal([ 'CGI directory', :string, nil ], args[0])
+      assert_equal([ 'CGI run command (optional)', :string, nil ], args[1])
+      assert_equal([ 'none parsed header', :bool, false ], args[2])
+      assert_equal([ 'pass CGI arguments', :bool, false ], args[3])
+      assert_equal([ 'pass authorization header', :bool, false ], args[4])
+      assert_equal([ 'environment variable name 1 (optional)', :string, nil ], args[5])
+      assert_equal([ 'environment variable value 1 (optional)', :string, nil ], args[6])
+      assert_equal([ 'environment variable name 2 (optional)', :string, nil ], args[7])
+      assert_equal([ 'environment variable value 2 (optional)', :string, nil ], args[8])
+      assert_equal([ 'environment variable name 3 (optional)', :string, nil ], args[9])
+      assert_equal([ 'environment variable value 3 (optional)', :string, nil ], args[10])
+      assert_equal([ 'environment variable name 4 (optional)', :string, nil ], args[11])
+      assert_equal([ 'environment variable value 4 (optional)', :string, nil ], args[12])
+      assert_equal([ 'environment variable name 5 (optional)', :string, nil ], args[13])
+      assert_equal([ 'environment variable value 5 (optional)', :string, nil ], args[14])
+    end
+
+    def test_new
+      doc = @factory.new('/', nil, false, false, false,
+			 nil, nil,
+			 nil, nil,
+			 nil, nil,
+			 nil, nil,
+			 nil, nil)
+      assert_instance_of(@loader.const_get('CGIDocument'), doc)
+    end
+  end
 end
