@@ -1,17 +1,19 @@
 #!/usr/local/bin/ruby
-# $Id: test_control.rb,v 1.1.1.1 2004/04/04 15:22:50 toki Exp $
+# $Id: test_control.rb,v 1.41 2004/10/20 14:37:36 toki Exp $
 
 require 'rubyunit'
 require 'wpm'
 require 'wpm/rucydriver'
+require 'rucy/properties'
 require 'rucy/control'
+require 'rucy/logger'
 require 'rucy/document'
 require 'rucy/passwd'
-require 'rucy/access'
-require 'rucy/auth'
 
 module TestRucy
   class TestControl < RUNIT::TestCase
+    include Rucy::LoggingLevel
+
     def setup
       # for Rucy::Properties class
       @properties = Hash.new
@@ -29,12 +31,17 @@ module TestRucy
       @set_params_last_names = nil
 
       # for Rucy::Server class
+      @start_time_call = 0
+      @restart_count_call = 0
+      @restart_time_call = 0
       @restart_call = 0
       @close_call = 0
       @restart_signal_call = 0
       @restart_signal = nil
       @port_call = 0
       @port = nil
+      @bind_address_call = 0
+      @bind_address = nil
       @timeout_call = 0
       @timeout = nil
       @keep_alive_call = 0
@@ -56,6 +63,9 @@ module TestRucy
       @subprocess_group_call = 0
       @subprocess_group = nil
       @add_logger_call = 0
+      @add_access_log_call = 0
+      @logging_access_call = 0
+      @messg_call = 0
       @set_alias_call = 0
       @set_alias_alist = Array.new
       @set_virtual_alias_call = 0
@@ -71,10 +81,13 @@ module TestRucy
       @pw_enc = Rucy::PasswordEncryptor.new
       @crypt_call = 0
 
-      # for Rucy::LibraryLoader class
-      @load_lib_call = 0
+      # for Rucy::DocumentLoader class
+      @loader = Rucy::DocumentLoader.new('../mod_docs')
+      @load_call = 0
       @load_errors_call = 0
-      @factory = Rucy::DocumentFactory.new
+      @factory_call = 0
+      @RemoteAddressAllowAccessFilter_call = 0
+      @BasicAuthFilter_call = 0
 
       @control = nil
     end
@@ -120,7 +133,9 @@ module TestRucy
       @params_last_names = names
       params = Hash.new
       for name in names
-	params[name] = @properties[name]
+	if (@properties.include? name) then
+	  params[name] = @properties[name]
+	end
       end
       params
     end
@@ -134,6 +149,21 @@ module TestRucy
       nil
     end
 
+    def start_time
+      @start_time_call += 1
+      Time.mktime(2004, 10, 13, 0, 44, 31)
+    end
+
+    def restart_count
+      @restart_count_call += 1
+      256
+    end
+
+    def restart_time
+      @restart_time_call += 1
+      Time.mktime(2004, 10, 13, 0, 45, 57)
+    end
+
     def restart
       @restart_call += 1
       nil
@@ -151,11 +181,20 @@ module TestRucy
       restart_signal
     end
 
+    def port
+      @port
+    end
+
     def port=(port)
       @port_call += 1
       @port = port
     end
 
+    def bind_address=(new_address)
+      @bind_address_call += 1
+      @bind_address = new_address
+    end
+
     def timeout=(timeout)
       @timeout_call += 1
       @timeout = timeout
@@ -211,6 +250,21 @@ module TestRucy
       nil
     end
 
+    def add_access_log(access_log)
+      @add_access_log_call += 1
+      nil
+    end
+
+    def logging_access(format)
+      @logging_access_call += 1
+      nil
+    end
+
+    def messg(level, messg, format=true)
+      @messg_call += 1
+      nil
+    end
+
     def set_alias(alias_path, orig_path)
       @set_alias_call += 1
       @set_alias_alist.push([ alias_path, orig_path ])
@@ -259,8 +313,9 @@ module TestRucy
       @pw_enc.crypt(password)
     end
 
-    def load_lib
-      @load_lib_call += 1
+    def load
+      @load_call += 1
+      @loader.load
       nil
     end
 
@@ -269,11 +324,25 @@ module TestRucy
       [ [ 'foo.rb', RuntimeError.new('HALO') ] ]
     end
 
+    def factory
+      @factory_call += 1
+      @loader.factory
+    end
+
+    def RemoteAddressAllowAccessFilter
+      @RemoteAddressAllowAccessFilter_call += 1
+      @loader.RemoteAddressAllowAccessFilter
+    end
+
+    def BasicAuthFilter
+      @BasicAuthFilter_call += 1
+      @loader.BasicAuthFilter
+    end
+
     def build_control
       @control = Rucy::Control.new
-      @control.lib_loader = self
+      @control.loader = self
       @control.pw_enc = self
-      @control.doc_factory = @factory
       @control.properties = self
       @control.page_path = 'nothing'
     end
@@ -311,201 +380,33 @@ module TestRucy
       build_control
       server_params = @control.server_params
       assert_equal(1, @params_call)
-      assert_equal([ 'server_type',
-		     'port',
-		     'timeout',
-		     'keep_alive',
-		     'max_requests',
-		     'queue_length',
-		     'messengers',
-		     'messenger_threads',
-		     'messenger_queue_length',
-		     'subprocess_user',
-		     'subprocess_group'
-		   ].sort, @params_last_names.sort)
-      assert_equal('multithread', server_params['server_type'])
-      assert_equal(8888, server_params['port'])
-      assert_equal(300, server_params['timeout'])
-      assert_equal(8, server_params['keep_alive'])
-      assert_equal(32, server_params['max_requests'])
-      assert_equal(16, server_params['queue_length'])
-      assert_equal(8, server_params['messengers'])
-      assert_equal(4, server_params['messenger_threads'])
-      assert_equal(4, server_params['messenger_queue_length'])
-      assert_equal('nobody', server_params['subprocess_user'])
-      assert_equal('nobody', server_params['subprocess_group'])
-
-      server_params['server_type'] = 'multiprocess'
-      server_params['port'] = 80
-      server_params['timeout'] = 60
-      server_params['keep_alive'] = 4
-      server_params['max_requests'] = 24
-      server_params['queue_length'] = 32
-      server_params['messengers'] = 16
-      server_params['messenger_threads'] = 8
-      server_params['messenger_queue_length'] = 8
-      server_params['subprocess_user'] = 'httpd'
-      server_params['subprocess_group'] = 'httpd'
       @control.set_server_params(server_params)
       assert_equal(1, @set_params_call)
-      assert_equal([ 'server_type',
-		     'port',
-		     'timeout',
-		     'keep_alive',
-		     'max_requests',
-		     'queue_length',
-		     'messengers',
-		     'messenger_threads',
-		     'messenger_queue_length',
-		     'subprocess_user',
-		     'subprocess_group'
-		   ].sort, @set_params_last_names.sort)
-      assert_equal('multiprocess', @properties['server_type'])
-      assert_equal(80, @properties['port'])
-      assert_equal(60, @properties['timeout'])
-      assert_equal(4, @properties['keep_alive'])
-      assert_equal(24, @properties['max_requests'])
-      assert_equal(32, @properties['queue_length'])
-      assert_equal(16, @properties['messengers'])
-      assert_equal(8, @properties['messenger_threads'])
-      assert_equal(8, @properties['messenger_queue_length'])
-      assert_equal('httpd', @properties['subprocess_user'])
-      assert_equal('httpd', @properties['subprocess_group'])
+      assert_exception(RuntimeError) {
+	@control.set_server_params(Hash.new)
+      }
     end
 
     def test_logging_params
       build_control
       logging_params = @control.logging_params
       assert_equal(1, @params_call)
-      assert_equal([ 'stdout_emerg',
-		     'stdout_alert',
-		     'stdout_crit',
-		     'stdout_err',
-		     'stdout_warning',
-		     'stdout_notice',
-		     'stdout_info',
-		     'stdout_debug',
-		     'logfiles'
-		   ].sort, @params_last_names.sort)
-      assert_equal(true, logging_params['stdout_emerg'])
-      assert_equal(true, logging_params['stdout_alert'])
-      assert_equal(true, logging_params['stdout_crit'])
-      assert_equal(true, logging_params['stdout_err'])
-      assert_equal(true, logging_params['stdout_warning'])
-      assert_equal(true, logging_params['stdout_notice'])
-      assert_equal(true, logging_params['stdout_info'])
-      assert_equal(false, logging_params['stdout_debug'])
-      assert_equal([], logging_params['logfiles'])
-      logging_params['stdout_debug'] = true
-      logging_params['logfiles'].push({ 'path' => 'server.log',
-					'log_emerg' => true,
-					'log_alert' => true,
-					'log_crit' => true,
-					'log_err' => true,
-					'log_warning' => true,
-					'log_notice' => true,
-					'log_info' => false,
-					'log_debug' => false
-				      })
-      logging_params['logfiles'].push({ 'path' => 'access.log',
-					'log_emerg' => false,
-					'log_alert' => false,
-					'log_crit' => false,
-					'log_err' => false,
-					'log_warning' => false,
-					'log_notice' => false,
-					'log_info' => true,
-					'log_debug' => false
-				      })
-      logging_params['logfiles'].push({ 'path' => 'debug.log',
-					'log_emerg' => false,
-					'log_alert' => false,
-					'log_crit' => false,
-					'log_err' => false,
-					'log_warning' => false,
-					'log_notice' => false,
-					'log_info' => false,
-					'log_debug' => true
-				      })
       @control.set_logging_params(logging_params)
       assert_equal(1, @set_params_call)
-      assert_equal([ 'stdout_emerg',
-		     'stdout_alert',
-		     'stdout_crit',
-		     'stdout_err',
-		     'stdout_warning',
-		     'stdout_notice',
-		     'stdout_info',
-		     'stdout_debug',
-		     'logfiles'
-		   ].sort, @set_params_last_names.sort)
-      assert_equal(true, @properties['stdout_emerg'])
-      assert_equal(true, @properties['stdout_alert'])
-      assert_equal(true, @properties['stdout_crit'])
-      assert_equal(true, @properties['stdout_err'])
-      assert_equal(true, @properties['stdout_warning'])
-      assert_equal(true, @properties['stdout_notice'])
-      assert_equal(true, @properties['stdout_info'])
-      assert_equal(true, @properties['stdout_debug'])
-      assert_equal(3, @properties['logfiles'].length)
-      assert_equal({ 'path' => 'server.log',
-		     'log_emerg' => true,
-		     'log_alert' => true,
-		     'log_crit' => true,
-		     'log_err' => true,
-		     'log_warning' => true,
-		     'log_notice' => true,
-		     'log_info' => false,
-		     'log_debug' => false
-		   }, @properties['logfiles'][0])
-      assert_equal({ 'path' => 'access.log',
-		     'log_emerg' => false,
-		     'log_alert' => false,
-		     'log_crit' => false,
-		     'log_err' => false,
-		     'log_warning' => false,
-		     'log_notice' => false,
-		     'log_info' => true,
-		     'log_debug' => false
-		   }, @properties['logfiles'][1])
-      assert_equal({ 'path' => 'debug.log',
-		     'log_emerg' => false,
-		     'log_alert' => false,
-		     'log_crit' => false,
-		     'log_err' => false,
-		     'log_warning' => false,
-		     'log_notice' => false,
-		     'log_info' => false,
-		     'log_debug' => true
-		   }, @properties['logfiles'][2])
+      assert_exception(RuntimeError) {
+	@control.set_logging_params(Hash.new)
+      }
     end
 
     def test_aliases
       build_control
       alias_list = @control.aliases
       assert_equal(1, @list_call)
-      assert_equal('aliases', @list_last_name)
-      assert(alias_list.empty?)
-      alias_list.push({ 'virtual_host' => nil,
-			'alias_path' => '/bar',
-			'orig_path' => '/foo'
-		      })
-      alias_list.push({ 'virtual_host' => 'foo',
-			'alias_path' => '/fuge',
-			'orig_path' => '/hoge'
-		      })
       @control.set_aliases(alias_list)
       assert_equal(1, @set_list_call)
-      assert_equal('aliases', @set_list_last_name)
-      assert_equal(2, @properties['aliases'].length)
-      assert_equal({ 'virtual_host' => nil,
-		     'alias_path' => '/bar',
-		     'orig_path' => '/foo'
-		   }, @properties['aliases'][0])
-      assert_equal({ 'virtual_host' => 'foo',
-		     'alias_path' => '/fuge',
-		     'orig_path' => '/hoge'
-		   }, @properties['aliases'][1])
+      assert_exception(RuntimeError) {
+	@control.set_aliases(Array.new)
+      }
     end
 
     def test_documents
@@ -521,7 +422,8 @@ module TestRucy
 			   ],
 			   'mount_path' => '/hello',
 			   'mount_mask' => nil,
-			   'virtual_host' => nil
+			   'virtual_host' => nil,
+			   'comment' => 'foo'
 			 })
       document_list.push({ 'document' => 'LocalFile',
 			   'arguments' => [
@@ -529,7 +431,8 @@ module TestRucy
 			   ],
 			   'mount_path' => '/~alice',
 			   'mount_mask' => nil,
-			   'virtual_host' => 'foo'
+			   'virtual_host' => 'foo',
+			   'comment' => 'bar'
 			 })
       @control.set_documents(document_list)
       assert_equal(1, @set_list_call)
@@ -542,7 +445,8 @@ module TestRucy
 		     ],
 		     'mount_path' => '/hello',
 		     'mount_mask' => nil,
-		     'virtual_host' => nil
+		     'virtual_host' => nil,
+		     'comment' => 'foo'
 		   }, @properties['documents'][0])
       assert_equal({ 'document' => 'LocalFile',
 		     'arguments' => [
@@ -550,7 +454,8 @@ module TestRucy
 		     ],
 		     'mount_path' => '/~alice',
 		     'mount_mask' => nil,
-		     'virtual_host' => 'foo'
+		     'virtual_host' => 'foo',
+		     'comment' => 'bar'
 		   }, @properties['documents'][1])
     end
 
@@ -566,7 +471,8 @@ module TestRucy
 			 ],
 			 'attach_path' => '/foo',
 			 'attach_mask' => %r"\.rhtml?($|/)",
-			 'virtual_host' => nil
+			 'virtual_host' => nil,
+			 'comment' => 'apple'
 		       })
       filter_list.push({ 'filter' => 'SSI',
 			 'arguments' => [
@@ -580,7 +486,8 @@ module TestRucy
 			 ],
 			 'attach_path' => '/bar',
 			 'attach_mask' => %r"\.shtml?($|/)",
-			 'virtual_host' => 'baz'
+			 'virtual_host' => 'baz',
+			 'comment' => 'banana'
 		       })
       @control.set_filters(filter_list)
       assert_equal(1, @set_list_call)
@@ -592,7 +499,8 @@ module TestRucy
 		     ],
 		     'attach_path' => '/foo',
 		     'attach_mask' => %r"\.rhtml?($|/)",
-		     'virtual_host' => nil
+		     'virtual_host' => nil,
+		     'comment' => 'apple'
 		   }, @properties['filters'][0])
       assert_equal({ 'filter' => 'SSI',
 		     'arguments' => [
@@ -606,20 +514,24 @@ module TestRucy
 		     ],
 		     'attach_path' => '/bar',
 		     'attach_mask' => %r"\.shtml?($|/)",
-		     'virtual_host' => 'baz'
+		     'virtual_host' => 'baz',
+		     'comment' => 'banana'
 		   }, @properties['filters'][1])
     end
 
     def test_default_server_settings
       build_control
       @control.server_setup(self)
-      assert_equal(3, @params_call) # for server parameters and logging parameters and admin parameters
+      assert_equal(4, @params_call) # for server parameters and logging parameters and access log parameters and admin parameters
       assert_equal(3, @list_call) # for (1) alias list and (2) document list and (3) filter list
-      assert_equal(1, @load_lib_call)
+      assert_equal(1, @load_call)
+      assert_equal(1, @factory_call)
       assert_equal(1, @restart_signal_call)
       assert_instance_of(Rucy::MultiThreadRestartSignal, @restart_signal)
       assert_equal(1, @port_call)
       assert_equal(8888, @port)
+      assert_equal(1, @bind_address_call)
+      assert_equal(nil, @bind_address)
       assert_equal(1, @timeout_call)
       assert_equal(300, @timeout)
       assert_equal(1, @keep_alive_call)
@@ -642,11 +554,28 @@ module TestRucy
       assert_equal('nobody', @subprocess_group)
       assert_equal(1, @add_logger_call)
       assert_equal(1, @close_hook_call)
+      assert_equal(1, @start_time_call)
+      assert_equal(2004, @control.start_time.year)
+      assert_equal(10, @control.start_time.month)
+      assert_equal(13, @control.start_time.day)
+      assert_equal(0, @control.start_time.hour)
+      assert_equal(44, @control.start_time.min)
+      assert_equal(31, @control.start_time.sec)
+      assert_equal(1, @restart_count_call)
+      assert_equal(256, @control.restart_count)
+      assert_equal(1, @restart_time_call)
+      assert_equal(2004, @control.restart_time.year)
+      assert_equal(10, @control.restart_time.month)
+      assert_equal(13, @control.restart_time.day)
+      assert_equal(0, @control.restart_time.hour)
+      assert_equal(45, @control.restart_time.min)
+      assert_equal(57, @control.restart_time.sec)
     end
 
     def test_modified_server_settings
       @properties['server_type'] = 'multiprocess'
       @properties['port'] = 80
+      @properties['bind_address'] = 'localhost'
       @properties['timeout'] = 60
       @properties['keep_alive'] = 4
       @properties['max_requests'] = 24
@@ -658,13 +587,16 @@ module TestRucy
       @properties['subprocess_group'] = 'httpd'
       build_control
       @control.server_setup(self)
-      assert_equal(3, @params_call) # for server parameters and logging parameters and admin parameters
+      assert_equal(4, @params_call) # for server parameters and logging parameters and access log parameters and admin parameters
       assert_equal(3, @list_call) # for (1) alias list and (2) document list and (3) filter list
-      assert_equal(1, @load_lib_call)
+      assert_equal(1, @load_call)
+      assert_equal(1, @factory_call)
       assert_equal(1, @restart_signal_call)
       assert_instance_of(Rucy::MultiProcessRestartSignal, @restart_signal)
       assert_equal(1, @port_call)
       assert_equal(80, @port)
+      assert_equal(1, @bind_address_call)
+      assert_equal('localhost', @bind_address)
       assert_equal(1, @timeout_call)
       assert_equal(60, @timeout)
       assert_equal(1, @keep_alive_call)
@@ -687,6 +619,22 @@ module TestRucy
       assert_equal('httpd', @subprocess_group)
       assert_equal(1, @add_logger_call)
       assert_equal(1, @close_hook_call)
+      assert_equal(1, @start_time_call)
+      assert_equal(2004, @control.start_time.year)
+      assert_equal(10, @control.start_time.month)
+      assert_equal(13, @control.start_time.day)
+      assert_equal(0, @control.start_time.hour)
+      assert_equal(44, @control.start_time.min)
+      assert_equal(31, @control.start_time.sec)
+      assert_equal(1, @restart_count_call)
+      assert_equal(256, @control.restart_count)
+      assert_equal(1, @restart_time_call)
+      assert_equal(2004, @control.restart_time.year)
+      assert_equal(10, @control.restart_time.month)
+      assert_equal(13, @control.restart_time.day)
+      assert_equal(0, @control.restart_time.hour)
+      assert_equal(45, @control.restart_time.min)
+      assert_equal(57, @control.restart_time.sec)
     end
 
     def test_admin_params
@@ -694,10 +642,10 @@ module TestRucy
       admin_params = @control.admin_params
       assert_equal(1, @params_call)
       assert_equal(%w[ admin_password admin_user japanese_handling localhost_only ], @params_last_names.sort)
-      assert_equal('admin', admin_params['admin_user'])
-      assert_equal('', admin_params['admin_password'])
-      assert_equal(true, admin_params['localhost_only'])
-      assert_equal(false, admin_params['japanese_handling'])
+      assert_equal('admin', admin_params.admin_user)
+      assert_equal('', admin_params.admin_password)
+      assert_equal(true, admin_params.localhost_only)
+      assert_equal(false, admin_params.japanese_handling)
     end
 
     def test_set_admin_params
@@ -705,17 +653,17 @@ module TestRucy
       admin_params = @control.admin_params
       assert_equal(1, @params_call)
       assert_equal(%w[ admin_password admin_user japanese_handling localhost_only ], @params_last_names.sort)
-      assert_equal('admin', admin_params['admin_user'])
-      assert_equal('', admin_params['admin_password'])
-      assert_equal(true, admin_params['localhost_only'])
-      admin_params['localhost_only'] = false
+      assert_equal('admin', admin_params.admin_user)
+      assert_equal('', admin_params.admin_password)
+      assert_equal(true, admin_params.localhost_only)
+      admin_params.localhost_only = false
       @control.set_admin_params(admin_params)
       assert_equal(1, @set_params_call)
       assert_equal(%w[ admin_password admin_user japanese_handling localhost_only ], @set_params_last_names.sort)
       assert_equal('admin', @properties['admin_user'])
       assert_equal('', @properties['admin_password'])
       assert_equal(true, @properties['localhost_only'])
-      admin_params['admin_password'] = @pw_enc.crypt('nazo')
+      admin_params.admin_password = @pw_enc.crypt('nazo')
       @control.set_admin_params(admin_params)
       assert_equal(2, @set_params_call)
       assert_equal(%w[ admin_password admin_user japanese_handling localhost_only ], @set_params_last_names.sort)
@@ -723,7 +671,7 @@ module TestRucy
       assert(@properties['admin_password'] != 'nazo')
       assert(@properties['admin_password'] == @pw_enc.crypt('nazo', @properties['admin_password']))
       assert_equal(true, @properties['localhost_only'])
-      admin_params['localhost_only'] = false
+      admin_params.localhost_only = false
       @control.set_admin_params(admin_params)
       assert_equal(3, @set_params_call)
       assert_equal(%w[ admin_password admin_user japanese_handling localhost_only ], @set_params_last_names.sort)
@@ -736,13 +684,17 @@ module TestRucy
     def test_ControlPanel_document
       build_control
       @control.server_setup(self)
+      assert_equal(1, @load_call)
+      assert_equal(1, @factory_call)
+      assert_equal(1, @RemoteAddressAllowAccessFilter_call)
+      assert_equal(0, @BasicAuthFilter_call)
       assert_equal(1, @mount_call)
       assert_equal(1, @attach_call)
       assert_equal(0, @virtual_mount_call)
       assert_equal(0, @virtual_attach_call)
       document = @folder.find('/control')[0]
       assert_instance_of(Rucy::FilterDocument, document)
-      assert_instance_of(Rucy::RemoteAddressAllowAccessFilter, document.filter)
+      assert_instance_of(@loader.RemoteAddressAllowAccessFilter, document.filter)
       assert_instance_of(Rucy::WebPageMakerDocument, document.document)
       assert_equal('/control', @folder.find('/control')[1])
     end
@@ -753,13 +705,17 @@ module TestRucy
       @properties['localhost_only'] = false
       build_control
       @control.server_setup(self)
+      assert_equal(1, @load_call)
+      assert_equal(1, @factory_call)
+      assert_equal(0, @RemoteAddressAllowAccessFilter_call)
+      assert_equal(1, @BasicAuthFilter_call)
       assert_equal(1, @mount_call)
       assert_equal(1, @attach_call)
       assert_equal(0, @virtual_mount_call)
       assert_equal(0, @virtual_attach_call)
       document = @folder.find('/control')[0]
       assert_instance_of(Rucy::FilterDocument, document)
-      assert_instance_of(Rucy::BasicAuth, document.filter)
+      assert_instance_of(@loader.BasicAuthFilter, document.filter)
       assert_instance_of(Rucy::WebPageMakerDocument, document.document)
       assert_equal('/control', @folder.find('/control')[1])
     end
@@ -770,59 +726,35 @@ module TestRucy
       @properties['localhost_only'] = true
       build_control
       @control.server_setup(self)
+      assert_equal(1, @load_call)
+      assert_equal(1, @factory_call)
+      assert_equal(1, @RemoteAddressAllowAccessFilter_call)
+      assert_equal(1, @BasicAuthFilter_call)
       assert_equal(1, @mount_call)
       assert_equal(2, @attach_call)
       assert_equal(0, @virtual_mount_call)
       assert_equal(0, @virtual_attach_call)
       document = @folder.find('/control')[0]
       assert_instance_of(Rucy::FilterDocument, document)
-      assert_instance_of(Rucy::BasicAuth, document.filter)
+      assert_instance_of(@loader.BasicAuthFilter, document.filter)
       document = document.document
       assert_instance_of(Rucy::FilterDocument, document)
-      assert_instance_of(Rucy::RemoteAddressAllowAccessFilter, document.filter)
+      assert_instance_of(@loader.RemoteAddressAllowAccessFilter, document.filter)
       assert_instance_of(Rucy::WebPageMakerDocument, document.document)
       assert_equal('/control', @folder.find('/control')[1])
     end
 
     def test_logging_setup
-      @properties['stdout_emerg'] = true
-      @properties['stdout_alert'] = true
-      @properties['stdout_crit'] = true
-      @properties['stdout_err'] = true
-      @properties['stdout_warning'] = true
-      @properties['stdout_notice'] = true
-      @properties['stdout_info'] = true
-      @properties['stdout_debug'] = true
+      @properties['stdout_logging_level'] = 'info'
       @properties['logfiles'] = [
 	{ 'path' => 'server.log',
-	  'log_emerg' => true,
-	  'log_alert' => true,
-	  'log_crit' => true,
-	  'log_err' => true,
-	  'log_warning' => true,
-	  'log_notice' => true,
-	  'log_info' => false,
-	  'log_debug' => false
+	  'logging_level' => 'notice'
 	},
 	{ 'path' => 'access.log',
-	  'log_emerg' => false,
-	  'log_alert' => false,
-	  'log_crit' => false,
-	  'log_err' => false,
-	  'log_warning' => false,
-	  'log_notice' => false,
-	  'log_info' => true,
-	  'log_debug' => false
+	  'logging_level' => 'info'
 	},
 	{ 'path' => 'debug.log',
-	  'log_emerg' => false,
-	  'log_alert' => false,
-	  'log_crit' => false,
-	  'log_err' => false,
-	  'log_warning' => false,
-	  'log_notice' => false,
-	  'log_info' => false,
-	  'log_debug' => true
+	  'logging_level' => 'debug'
 	}
       ]
 
@@ -849,14 +781,7 @@ module TestRucy
     def test_logging_errors
       @properties['logfiles'] = [
 	{ 'path' => 'detarame/server.log',
-	  'log_emerg' => true,
-	  'log_alert' => true,
-	  'log_crit' => true,
-	  'log_err' => true,
-	  'log_warning' => true,
-	  'log_notice' => true,
-	  'log_info' => false,
-	  'log_debug' => false
+	  'logging_level' => 'notice'
 	}
       ]
       build_control
@@ -916,101 +841,65 @@ module TestRucy
 	},
 	{ 'document' => 'LocalFile',
 	  'arguments' => [
-	    '/home/alice/public_html'
+	    '/home/alice/public_html',
+	    nil
 	  ],
 	  'mount_path' => '/~alice',
 	  'mount_mask' => nil,
 	  'virtual_host' => 'foo'
 	}
       ]
-
-      page = Object.new
-      class << page
-	include RUNIT::Assert
-
-	def doc_name
-	  'Page'
-	end
-
-	def doc_args
-	  [ [ 'content', :text, nil ],
-	    [ 'content-type', :string, 'text/html' ]
-	  ]
-	end
-
-	def new(content, content_type)
-	  assert_equal("Hello world.\n", content)
-	  assert_equal('text/html', content_type)
-	  :dummy_page_document
-	end
-      end
-      @factory.add_document(page)
-
-      local = Object.new
-      class << local
-	include RUNIT::Assert
-
-	def doc_name
-	  'LocalFile'
-	end
-
-	def doc_args
-	  [ [ 'local path', :string, nil ] ]
-	end
-
-	def new(local_path)
-	  assert_equal('/home/alice/public_html', local_path)
-	  :dummy_local_document
-	end
-      end
-      @factory.add_document(local)
-
       build_control
       @control.server_setup(self)
       assert_equal(2, @mount_call)
       assert_equal(1, @virtual_mount_call)
-      assert_equal(:dummy_page_document, @folder.find('/hello')[0])
+      assert_instance_of(@loader.PageDocument, @folder.find('/hello')[0])
       assert_equal('/hello', @folder.find('/hello')[1])
-      assert_equal(:dummy_local_document, @folder.virtual_find('foo:8888', '/~alice')[0])
+      assert_instance_of(@loader.LocalFileDocument, @folder.virtual_find('foo:8888', '/~alice')[0])
       assert_equal('/~alice', @folder.virtual_find('foo:8888', '/~alice')[1])
     end
 
-    def test_document_setup_error
-      error = Object.new
-      def error.doc_name
-	'TestError'
-      end
-      def error.doc_args
-	[]
-      end
-      def error.new
-	raise 'exception on creating a document.'
-      end
-      @factory.add_document(error)
+#     def test_document_setup_error
+#       error = Object.new
+#       class << error
+# 	def set_doc_option(option)
+# 	end
 
-      @properties['documents'] = [
-	{ 'document' => 'TestError',
-	  'arguments' => [],
-	  'mount_path' => '/error',
-	  'mount_mask' => nil,
-	  'virtual_host' => 'foo'
-	}
-      ]
+# 	def doc_name
+# 	  'TestError'
+# 	end
 
-      build_control
-      @control.server_setup(self)
-      assert_equal(1, @mount_call)
-      assert_equal(0, @virtual_mount_call)
-      assert_nil(@folder.find('/error'))
-      assert_equal(1, @control.doc_errors.length)
-      assert_equal('TestError', @control.doc_errors[0][:document])
-      assert_equal([], @control.doc_errors[0][:arguments])
-      assert_equal('/error', @control.doc_errors[0][:mount_path])
-      assert_nil(@control.doc_errors[0][:mount_mask])
-      assert_equal('foo', @control.doc_errors[0][:virtual_host])
-      assert_instance_of(RuntimeError, @control.doc_errors[0][:exception])
-      assert_equal('exception on creating a document.', @control.doc_errors[0][:exception].message)
-    end
+# 	def doc_args
+# 	  []
+# 	end
+
+# 	def new
+# 	  raise 'exception on creating a document.'
+# 	end
+#       end
+#       @factory.add_document(error)
+#       @properties['documents'] = [
+# 	{ 'document' => 'TestError',
+# 	  'arguments' => [],
+# 	  'mount_path' => '/error',
+# 	  'mount_mask' => nil,
+# 	  'virtual_host' => 'foo'
+# 	}
+#       ]
+#       build_control
+#       @control.server_setup(self)
+#       assert_equal(1, @mount_call)
+#       assert_equal(0, @virtual_mount_call)
+#       assert_nil(@folder.find('/error'))
+#       assert_equal(1, @control.doc_errors.length)
+#       assert_equal('TestError', @control.doc_errors[0][:document])
+#       assert_equal([], @control.doc_errors[0][:arguments])
+#       assert_equal('/error', @control.doc_errors[0][:mount_path])
+#       assert_nil(@control.doc_errors[0][:mount_mask])
+#       assert_equal('foo', @control.doc_errors[0][:virtual_host])
+#       assert_instance_of(RuntimeError, @control.doc_errors[0][:exception])
+#       assert_equal('exception on creating a document.', @control.doc_errors[0][:exception].message)
+#     end
 
     def test_filter_setup
       @properties['documents'] = [
@@ -1033,196 +922,89 @@ module TestRucy
 	  'virtual_host' => 'baz'
 	}
       ]
-
-      page = Object.new
-      class << page
-	include RUNIT::Assert
-
-	def doc_name
-	  'Page'
-	end
-
-	def doc_args
-	  [ [ 'content', :text, nil ],
-	    [ 'content-type', :string, 'text/html' ]
-	  ]
-	end
-
-	def new(content, content_type)
-	  assert(content == "Hello world.\n" ||
-		 content == "Hello virtual world.\n")
-	  assert_equal('text/html', content_type)
-	  :dummy_page_document
-	end
-      end
-      @factory.add_document(page)
-
       @properties['filters'] = [
 	{ 'filter' => 'ERB',
-	  'arguments' => [
-	    1
-	  ],
+	  'arguments' => [ '1', false ],
 	  'attach_path' => '/foo',
 	  'attach_mask' => %r"\.rhtml?($|/)",
 	  'virtual_host' => nil
 	},
-	{ 'filter' => 'SSI',
-	  'arguments' => [
-	    { 'config' => true,
-	      'include' => true,
-	      'echo' => true,
-	      'fsize' => true,
-	      'flastmod' => true,
-	      'exec' => true
-	    }
-	  ],
+	{ 'filter' => 'ERB',
+	  'arguments' => [ '1', true ],
 	  'attach_path' => '/bar',
-	  'attach_mask' => %r"\.shtml?($|/)",
+	  'attach_mask' => %r"\.rhtml?($|/)",
 	  'virtual_host' => 'baz'
 	}
       ]
-
-      erb = Object.new
-      class << erb
-	include RUNIT::Assert
-
-	def filter_name
-	  'ERB'
-	end
-
-	def filter_args
-	  [ [ 'safe level', :number, 1 ] ]
-	end
-
-	def new(safe_level)
-	  assert_equal(1, safe_level)
-	  :dummy_ERB_document
-	end
-      end
-      @factory.add_filter(erb)
-
-      ssi = Object.new
-      class << ssi
-	include RUNIT::Assert
-
-	def filter_name
-	  'SSI'
-	end
-
-	def filter_args
-	  [ [ 'SSI commands', :checkset,
-	      [ [ 'config',   true ],
-		[ 'include',  true ],
-		[ 'echo',     true ],
-		[ 'fsize',    true ],
-		[ 'flastmod', true ],
-		[ 'exec',     true ]
-	      ]
-	    ]
-	  ]
-	end
-
-	def new(cmds)
-	  assert_equal({ 'config'   => true,
-			 'include'  => true,
-			 'echo'     => true,
-			 'fsize'    => true,
-			 'flastmod' => true,
-			 'exec'     => true
-		       }, cmds)
-	  :dummy_SSI_document
-	end
-      end
-      @factory.add_filter(ssi)
-
       build_control
       @control.server_setup(self)
       assert_equal(2, @attach_call)
       assert_equal(1, @virtual_attach_call)
-      assert_equal(:dummy_ERB_document, @folder.find('/foo/index.rhtml')[0].filter)
+      assert_instance_of(@loader.ERBFilter, @folder.find('/foo/index.rhtml')[0].filter)
       assert_equal('/foo', @folder.find('/foo/index.rhtml')[1])
-      assert_equal(:dummy_SSI_document, @folder.virtual_find('baz:8888', '/bar/index.shtml')[0].filter)
+      assert_instance_of(@loader.ERBFilter, @folder.virtual_find('baz:8888', '/bar/index.rhtml')[0].filter)
       assert_equal('/bar', @folder.virtual_find('baz:8888', '/bar/index.shtml')[1])
     end
 
-    def test_filter_setup_error
-      @properties['documents'] = [
-	{ 'document' => 'Page',
-	  'arguments' => [
-	    "Hello world.\n",
-	    'text/html'
-	  ],
-	  'mount_path' => '/',
-	  'mount_mask' => nil,
-	  'virtual_host' => nil
-	},
-	{ 'document' => 'Page',
-	  'arguments' => [
-	    "Hello virtual world.\n",
-	    'text/html'
-	  ],
-	  'mount_path' => '/',
-	  'mount_mask' => nil,
-	  'virtual_host' => 'baz'
-	}
-      ]
+#     def test_filter_setup_error
+#       @properties['documents'] = [
+# 	{ 'document' => 'Page',
+# 	  'arguments' => [
+# 	    "Hello world.\n",
+# 	    'text/html'
+# 	  ],
+# 	  'mount_path' => '/',
+# 	  'mount_mask' => nil,
+# 	  'virtual_host' => nil
+# 	},
+# 	{ 'document' => 'Page',
+# 	  'arguments' => [
+# 	    "Hello virtual world.\n",
+# 	    'text/html'
+# 	  ],
+# 	  'mount_path' => '/',
+# 	  'mount_mask' => nil,
+# 	  'virtual_host' => 'baz'
+# 	}
+#       ]
+#       error = Object.new
+#       class << error
+# 	def set_filter_option(option)
+# 	end
 
-      page = Object.new
-      class << page
-	include RUNIT::Assert
+# 	def filter_name
+# 	  'TestError'
+# 	end
 
-	def doc_name
-	  'Page'
-	end
+# 	def filter_args
+# 	  []
+# 	end
 
-	def doc_args
-	  [ [ 'content', :text, nil ],
-	    [ 'content-type', :string, 'text/html' ]
-	  ]
-	end
-
-	def new(content, content_type)
-	  assert(content == "Hello world.\n" ||
-		 content == "Hello virtual world.\n")
-	  assert_equal('text/html', content_type)
-	  :dummy_page_document
-	end
-      end
-      @factory.add_document(page)
-
-      error = Object.new
-      def error.filter_name
-	'TestError'
-      end
-      def error.filter_args
-	[]
-      end
-      def error.new
-	raise 'exception on creating a filter.'
-      end
-      @factory.add_filter(error)
-
-      @properties['filters'] = [
-	{ 'filter' => 'TestError',
-	  'arguments' => [],
-	  'attach_path' => '/error',
-	  'attach_mask' => nil,
-	  'virtual_host' => 'foo'
-	}
-      ]
-
-      build_control
-      @control.server_setup(self)
-      assert_equal(1, @attach_call)
-      assert_equal(0, @virtual_attach_call)
-      assert_equal(1, @control.filter_errors.length)
-      assert_equal('TestError', @control.filter_errors[0][:filter])
-      assert_equal([], @control.filter_errors[0][:arguments])
-      assert_equal('/error', @control.filter_errors[0][:attach_path])
-      assert_nil(@control.filter_errors[0][:attach_mask])
-      assert_equal('foo', @control.filter_errors[0][:virtual_host])
-      assert_instance_of(RuntimeError, @control.filter_errors[0][:exception])
-      assert_equal('exception on creating a filter.', @control.filter_errors[0][:exception].message)
-    end
+# 	def new
+# 	  raise 'exception on creating a filter.'
+# 	end
+#       end
+#       @factory.add_filter(error)
+#       @properties['filters'] = [
+# 	{ 'filter' => 'TestError',
+# 	  'arguments' => [],
+# 	  'attach_path' => '/error',
+# 	  'attach_mask' => nil,
+# 	  'virtual_host' => 'foo'
+# 	}
+#       ]
+#       build_control
+#       @control.server_setup(self)
+#       assert_equal(1, @attach_call)
+#       assert_equal(0, @virtual_attach_call)
+#       assert_equal(1, @control.filter_errors.length)
+#       assert_equal('TestError', @control.filter_errors[0][:filter])
+#       assert_equal([], @control.filter_errors[0][:arguments])
+#       assert_equal('/error', @control.filter_errors[0][:attach_path])
+#       assert_nil(@control.filter_errors[0][:attach_mask])
+#       assert_equal('foo', @control.filter_errors[0][:virtual_host])
+#       assert_instance_of(RuntimeError, @control.filter_errors[0][:exception])
+#       assert_equal('exception on creating a filter.', @control.filter_errors[0][:exception].message)
+#     end
   end
 end
