@@ -1,6 +1,7 @@
 #!/usr/local/bin/ruby
-# $Id: test_document.rb,v 1.2 2004/04/16 03:24:05 toki Exp $
+# $Id: test_document.rb,v 1.6 2004/10/02 00:42:02 toki Exp $
 
+require 'singleton'
 require 'rubyunit'
 require 'pseudo_req_res'
 require 'rucy/request'
@@ -158,13 +159,11 @@ module TestRucy
       @doc_name_call = 0
       @doc_args_call = 0
       @set_doc_option_call = 0
-      @set_doc_option_last_name = nil
-      @set_doc_option_last_value = nil
+      @set_doc_option_value = nil
       @filter_name_call = 0
       @filter_args_call = 0
       @set_filter_option_call = 0
-      @set_filter_option_last_name = nil
-      @set_filter_option_last_value = nil
+      @set_filter_option_value = nil
       @new_call = 0
       @new_string_value = nil
       @new_text_value = nil
@@ -192,6 +191,12 @@ module TestRucy
       nil
     end
 
+    def set_doc_option(option)
+      @set_doc_option_call += 1
+      @set_doc_option_value = option
+      nil
+    end
+
     def filter_name
       @filter_name_call += 1
       'test_filter'
@@ -212,6 +217,12 @@ module TestRucy
       nil
     end
 
+    def set_filter_option(option)
+      @set_filter_option_call += 1
+      @set_filter_option_value = option
+      nil
+    end
+
     def new(string, text, number)
       @new_call += 1
       @new_string_value = string
@@ -220,6 +231,20 @@ module TestRucy
       :test
     end
 
+    def test_setup
+      @factory.add_option(:foo, 'apple')
+      @factory.add_option(:bar, 'banana')
+      @factory.add_document(self)
+      @factory.add_filter(self)
+      @factory.setup
+      assert_equal(1, @set_doc_option_call)
+      assert_equal('apple', @set_doc_option_value[:foo])
+      assert_equal('banana', @set_doc_option_value[:bar])
+      assert_equal(1, @set_filter_option_call)
+      assert_equal('apple', @set_filter_option_value[:foo])
+      assert_equal('banana', @set_filter_option_value[:bar])
+    end
+
     def test_has_document?
       assert(! (@factory.has_document? 'test_document'))
       @factory.add_document(self)
@@ -265,14 +290,6 @@ module TestRucy
       assert_equal(:test, dummy_doc)
     end
 
-    def test_set_doc_option
-      @factory.add_document(self)
-      @factory.set_doc_option('test_document', :test_message, "Hello world.")
-      assert_equal(1, @set_doc_option_call)
-      assert_equal(:test_message, @set_doc_option_last_name)
-      assert_equal("Hello world.", @set_doc_option_last_value)
-    end
-
     def test_has_filter?
       assert(! (@factory.has_filter? 'test_filter'))
       @factory.add_filter(self)
@@ -317,23 +334,188 @@ module TestRucy
       assert_equal(0, @new_number_value)
       assert_equal(:test, dummy_filter)
     end
+  end
 
-    def test_set_filter_option
-      @factory.add_filter(self)
-      @factory.set_filter_option('test_filter', :test_message, "Hello world.")
-      assert_equal(1, @set_filter_option_call)
-      assert_equal(:test_message, @set_filter_option_last_name)
-      assert_equal("Hello world.", @set_filter_option_last_value)
+  class TestDocumentLoader < RUNIT::TestCase
+    def setup
+      Dir.mkdir('mod_docs_for_test')
+
+      # target
+      @loader = Rucy::DocumentLoader.new('mod_docs_for_test')
+    end
+
+    def teardown
+      Dir.foreach('mod_docs_for_test') do |path|
+	if (! (File.directory? path)) then
+	  File.delete("mod_docs_for_test/#{path}")
+	end
+      end
+      Dir.delete('mod_docs_for_test')
+    end
+
+    def test_load_docs
+      File.open('mod_docs_for_test/foo.rb', 'w') {|output|
+	output.print "include Rucy\n"
+	output.print "class FooDocument < Document\n"
+	output.print "  def self.doc_name\n"
+	output.print "    'Foo'\n"
+	output.print "  end\n"
+	output.print "  def self.doc_args\n"
+	output.print "    []\n"
+	output.print "  end\n"
+	output.print "end\n"
+      }
+      File.open('mod_docs_for_test/bar.rb', 'w') {|output|
+	output.print "include Rucy\n"
+	output.print "class BarFilter < Filter\n"
+	output.print "  def self.filter_name\n"
+	output.print "    'Bar'\n"
+	output.print "  end\n"
+	output.print "  def self.filter_args\n"
+	output.print "    []\n"
+	output.print "  end\n"
+	output.print "end\n"
+      }
+      @loader.load
+      assert_instance_of(Class, @loader.doc_get('FooDocument'))
+      assert_equal(@loader.doc_get('FooDocument'), @loader.FooDocument)
+      assert_instance_of(Class, @loader.filter_get('BarFilter'))
+      assert_equal(@loader.filter_get('BarFilter'), @loader.BarFilter)
+      factory = @loader.factory
+      factory.setup
+      assert_equal([ 'Foo' ], factory.doc_names)
+      assert_equal([], factory.doc_args('Foo'))
+      assert_instance_of(@loader.FooDocument, factory.doc_build('Foo', []))
+      assert_equal([ 'Bar' ], factory.filter_names)
+      assert_equal([], factory.filter_args('Bar'))
+      assert_instance_of(@loader.BarFilter, factory.filter_build('Bar', []))
+    end
+
+    def test_load_factories
+      File.open('mod_docs_for_test/foo.rb', 'w') {|output|
+	output.print "include Rucy\n"
+	output.print "class FooDocument < Document\n"
+	output.print "end\n"
+	output.print "class FooDocumentFactory\n"
+	output.print "  include Singleton\n"
+	output.print "  def set_doc_option(opts)\n"
+	output.print "  end\n"
+	output.print "  def doc_name\n"
+	output.print "    'Foo'\n"
+	output.print "  end\n"
+	output.print "  def doc_args\n"
+	output.print "    []\n"
+	output.print "  end\n"
+	output.print "  def new\n"
+	output.print "    FooDocument.new\n"
+	output.print "  end\n"
+	output.print "end\n"
+      }
+      File.open('mod_docs_for_test/bar.rb', 'w') {|output|
+	output.print "include Rucy\n"
+	output.print "class BarFilter < Filter\n"
+	output.print "end\n"
+	output.print "class BarFilterFactory\n"
+	output.print "  include Singleton\n"
+	output.print "  def set_filter_option(opts)\n"
+	output.print "  end\n"
+	output.print "  def filter_name\n"
+	output.print "    'Bar'\n"
+	output.print "  end\n"
+	output.print "  def filter_args\n"
+	output.print "    []\n"
+	output.print "  end\n"
+	output.print "  def new\n"
+	output.print "    BarFilter.new\n"
+	output.print "  end\n"
+	output.print "end\n"
+      }
+      @loader.load
+      assert(@loader.load_errors.empty?)
+      assert_instance_of(Class, @loader.doc_get('FooDocument'))
+      assert_equal(@loader.doc_get('FooDocument'), @loader.FooDocument)
+      assert_instance_of(Class, @loader.filter_get('BarFilter'))
+      assert_equal(@loader.filter_get('BarFilter'), @loader.BarFilter)
+      factory = @loader.factory
+      factory.setup
+      assert_equal([ 'Foo' ], factory.doc_names)
+      assert_equal([], factory.doc_args('Foo'))
+      assert_instance_of(@loader.FooDocument, factory.doc_build('Foo', []))
+      assert_equal([ 'Bar' ], factory.filter_names)
+      assert_equal([], factory.filter_args('Bar'))
+      assert_instance_of(@loader.BarFilter, factory.filter_build('Bar', []))
+    end
+
+    def test_load2
+      File.open('mod_docs_for_test/foo.rb', 'w') {|output|
+	output.print "include Rucy\n"
+	output.print "class FooDocument < Document\n"
+	output.print "  def self.doc_name\n"
+	output.print "    'Foo'\n"
+	output.print "  end\n"
+	output.print "  def self.doc_args\n"
+	output.print "    []\n"
+	output.print "  end\n"
+	output.print "end\n"
+      }
+      File.open('mod_docs_for_test/bar.rb', 'w') {|output|
+	output.print "include Rucy\n"
+	output.print "class BarFilter < Filter\n"
+	output.print "  def self.filter_name\n"
+	output.print "    'Bar'\n"
+	output.print "  end\n"
+	output.print "  def self.filter_args\n"
+	output.print "    []\n"
+	output.print "  end\n"
+	output.print "end\n"
+      }
+      @loader.load
+      assert(@loader.load_errors.empty?)
+      fooDocument = @loader.FooDocument
+      barFilter = @loader.BarFilter
+      factory = @loader.factory
+      factory.setup
+      foo = factory.doc_build('Foo', [])
+      bar = factory.filter_build('Bar', [])
+      assert_instance_of(fooDocument, foo)
+      assert_instance_of(barFilter, bar)
+      @loader.load
+      assert(@loader.FooDocument != fooDocument)
+      assert(@loader.BarFilter != barFilter)
+      factory2 = @loader.factory
+      assert(factory2 != factory)
+      foo2 = factory2.doc_build('Foo', [])
+      bar2 = factory2.filter_build('Bar', [])
+      assert(! (foo2.instance_of? fooDocument))
+      assert(! (bar2.instance_of? barFilter))
+    end
+
+    def test_load_error
+      File.open('mod_docs_for_test/foo.rb', 'w') {|output|
+	output.print "raise 'foo'\n"
+      }
+      File.open('mod_docs_for_test/bar.rb', 'w') {|output|
+	output.print "require 'no_load'\n"
+      }
+      @loader.load
+      assert_equal([ 'mod_docs_for_test/foo.rb',
+		     'mod_docs_for_test/bar.rb'
+		   ].sort,
+		   @loader.load_errors.map{|k,v| k }.sort)
+      assert_instance_of(RuntimeError,
+			 @loader.load_errors.assoc('mod_docs_for_test/foo.rb')[1])
+      assert_instance_of(LoadError,
+			 @loader.load_errors.assoc('mod_docs_for_test/bar.rb')[1])
     end
   end
 
-  class TestPageDocument < RUNIT::TestCase
+  class TestPage < RUNIT::TestCase
     include PseudoLogger
     include PseudoRequestResponse
 
     def setup
       super
-      @page = Rucy::PageDocument.new("<html>Hello world.</html>\n")
+      @page = Rucy::Page.new("<html>Hello world.</html>\n")
     end
 
     def test_GET
@@ -356,7 +538,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::PageDocument/)
+      assert_match(@log_debug, /enter document: Rucy::Page/)
     end
 
     def test_HEAD
@@ -379,7 +561,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::PageDocument/)
+      assert_match(@log_debug, /enter document: Rucy::Page/)
     end
   end
 
