@@ -1,17 +1,20 @@
 #!/usr/local/bin/ruby
-# $Id: test_local.rb,v 1.1.1.1 2004/04/04 15:22:50 toki Exp $
+# $Id: test_local.rb,v 1.4 2004/10/02 01:00:18 toki Exp $
 
 require 'rubyunit'
 require 'time'
 require 'pseudo_req_res'
 require 'rucy/request'
 require 'rucy/response'
-require 'rucy/local'
+require 'rucy/loader'
 
 module TestRucy
   class TestContentTypeResolver < RUNIT::TestCase
     def setup
-      @resolver = Rucy::ContentTypeResolver.new
+      @loader = Rucy::SimpleLoader.new
+      @loader.load('../mod_docs/local.rb')
+      @ContentTypeResolver = @loader.const_get('ContentTypeResolver')
+      @resolver = @ContentTypeResolver.new
     end
 
     def test_content_type
@@ -44,13 +47,16 @@ module TestRucy
     end
   end
 
-  class TestLocalDirectoryDocument < RUNIT::TestCase
+  class TestLocalDirectory < RUNIT::TestCase
     include PseudoLogger
     include PseudoRequestResponse
 
     def setup
       super
-      @dir_doc = Rucy::LocalDirectoryDocument.new('test')
+      @loader = Rucy::SimpleLoader.new
+      @loader.load('../mod_docs/local.rb')
+      @LocalDirectory = @loader.const_get('LocalDirectory')
+      @dir_doc = @LocalDirectory.new('test')
       Dir.mkdir('test')
       Dir.mkdir('test/subdir')
       File.open('test/foo', 'w').close
@@ -79,7 +85,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalDirectoryDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalDirectory/)
     end
 
     def test_HEAD
@@ -99,7 +105,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalDirectoryDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalDirectory/)
     end
 
     def test_not_allowed
@@ -124,7 +130,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalDirectoryDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalDirectory/)
     end
 
     def test_not_found
@@ -148,7 +154,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalDirectoryDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalDirectory/)
     end
 
     def test_subdir
@@ -167,7 +173,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalDirectoryDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalDirectory/)
     end
 
     def test_file
@@ -191,7 +197,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalDirectoryDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalDirectory/)
     end
   end
 
@@ -215,7 +221,10 @@ module TestRucy
 	output.print DATA
       }
 
-      @local_doc = Rucy::LocalFileDocument.new('test')
+      @loader = Rucy::SimpleLoader.new
+      @loader.load('../mod_docs/local.rb')
+      @LocalFileDocument = @loader.const_get('LocalFileDocument')
+      @local_doc = @LocalFileDocument.new('test')
     end
 
     def teardown
@@ -245,7 +254,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_HEAD_root_200_OK
@@ -268,7 +277,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_POST_root_405_Method_Not_Allowed
@@ -293,7 +302,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_html_200_OK
@@ -316,7 +325,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_HEAD_html_200_OK
@@ -339,7 +348,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_POST_html_405_Method_Not_Allowed
@@ -364,7 +373,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_nothing_404_Not_Found
@@ -388,7 +397,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_subdir_301_Moved_Permanently
@@ -409,7 +418,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_subdir_200_OK
@@ -431,7 +440,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_subdir_file_200_OK
@@ -453,7 +462,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_illegal_path_403_Forbidden
@@ -477,7 +486,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert_match(@log_info, /illegal path: /)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_if_modified_since_304_Not_Modified
@@ -500,7 +509,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_if_modified_since_200_OK
@@ -526,7 +535,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_if_unmodified_since_412_Precondition_Failed
@@ -550,7 +559,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_if_unmodified_since_200_OK
@@ -575,7 +584,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_range_206_Partial_Content
@@ -600,7 +609,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_range_416_Requested_Range_Not_Satisfiable
@@ -622,7 +631,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_range_200_OK
@@ -647,7 +656,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_range_if_modified_since_304_Not_Modified
@@ -671,7 +680,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_range_if_modified_since_206_Partial_Content
@@ -699,7 +708,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_range_if_unmodified_since_412_Precondition_Failed
@@ -724,7 +733,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_range_if_unmodified_since_206_Partial_Content
@@ -751,7 +760,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_if_range_206_Partial_Content
@@ -778,7 +787,7 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
     end
 
     def test_GET_if_range_200_OK
@@ -806,7 +815,50 @@ module TestRucy
       assert(@log_warning.empty?)
       assert(@log_notice.empty?)
       assert(@log_info.empty?)
-      assert_match(@log_debug, /enter document: Rucy::LocalFileDocument/)
+      assert_match(@log_debug, /enter document: \S+::LocalFileDocument/)
+    end
+  end
+
+  class TestLocalFileDocumentFactory < RUNIT::TestCase
+    def setup
+      @loader = Rucy::SimpleLoader.new
+      @loader.load('../mod_docs/local.rb')
+      @LocalFileDocumentFactory = @loader.const_get('LocalFileDocumentFactory')
+      @factory = @LocalFileDocumentFactory.instance
+    end
+
+    def test_set_doc_option
+      @factory.set_doc_option(Hash.new)
+    end
+
+    def tset_doc_name
+      assert_equal('LocalFile', @factory.doc_name)
+    end
+
+    def test_doc_args
+      args = @factory.doc_args
+      assert_equal([ 'local path', :string, nil ], args[0])
+      assert_equal([ 'default content-type (optional)', :string, nil ], args[1])
+      assert_equal([ 'suffix 1 (optional)', :string, nil ], args[2])
+      assert_equal([ 'content-type 1 (optional)', :string, nil ], args[3])
+      assert_equal([ 'suffix 2 (optional)', :string, nil ], args[4])
+      assert_equal([ 'content-type 2 (optional)', :string, nil ], args[5])
+      assert_equal([ 'suffix 3 (optional)', :string, nil ], args[6])
+      assert_equal([ 'content-type 3 (optional)', :string, nil ], args[7])
+      assert_equal([ 'suffix 4 (optional)', :string, nil ], args[8])
+      assert_equal([ 'content-type 4 (optional)', :string, nil ], args[9])
+      assert_equal([ 'suffix 5 (optional)', :string, nil ], args[10])
+      assert_equal([ 'content-type 5 (optional)', :string, nil ], args[11])
+    end
+
+    def test_new
+      doc = @factory.new('/', nil,
+			 nil, nil,
+			 nil, nil,
+			 nil, nil,
+			 nil, nil,
+			 nil, nil)
+      assert_instance_of(@loader.const_get('LocalFileDocument'), doc)
     end
   end
 end
